@@ -9,7 +9,7 @@
  * @subpackage Administration
  */
 
-_deprecated_file( basename(__FILE__), '2.5', 'wp-admin/includes/admin.php' );
+_deprecated_file( basename(__FILE__), '2.5.0', 'wp-admin/includes/admin.php' );
 
 /** WordPress Administration API: Includes all Administration functions. */
 require_once(ABSPATH . 'wp-admin/includes/admin.php');
