@@ -15,7 +15,6 @@ class WP_Filesystem_Base {
 	/**
 	 * Whether to display debug data for the connection.
 	 *
-	 * @access public
 	 * @since 2.5.0
 	 * @var bool
 	 */
@@ -32,20 +31,23 @@ class WP_Filesystem_Base {
 	/**
 	 * The Access method of the current connection, Set automatically.
 	 *
-	 * @access public
 	 * @since 2.5.0
 	 * @var string
 	 */
 	public $method = '';
 
+	/**
+	 * @var WP_Error
+	 */
 	public $errors = null;
 
+	/**
+	 */
 	public $options = array();
 
 	/**
 	 * Return the path on the remote filesystem of ABSPATH.
 	 *
-	 * @access public
 	 * @since 2.7.0
 	 *
 	 * @return string The location of the remote path.
@@ -61,7 +63,6 @@ class WP_Filesystem_Base {
 	/**
 	 * Return the path on the remote filesystem of WP_CONTENT_DIR.
 	 *
-	 * @access public
 	 * @since 2.7.0
 	 *
 	 * @return string The location of the remote path.
@@ -73,7 +74,6 @@ class WP_Filesystem_Base {
 	/**
 	 * Return the path on the remote filesystem of WP_PLUGIN_DIR.
 	 *
-	 * @access public
 	 * @since 2.7.0
 	 *
 	 * @return string The location of the remote path.
@@ -85,7 +85,6 @@ class WP_Filesystem_Base {
 	/**
 	 * Return the path on the remote filesystem of the Themes Directory.
 	 *
-	 * @access public
 	 * @since 2.7.0
 	 *
 	 * @param string $theme The Theme stylesheet or template for the directory.
@@ -104,7 +103,6 @@ class WP_Filesystem_Base {
 	/**
 	 * Return the path on the remote filesystem of WP_LANG_DIR.
 	 *
-	 * @access public
 	 * @since 3.2.0
 	 *
 	 * @return string The location of the remote path.
@@ -116,7 +114,6 @@ class WP_Filesystem_Base {
 	/**
 	 * Locate a folder on the remote filesystem.
 	 *
-	 * @access public
 	 * @since 2.5.0
 	 * @deprecated 2.7.0 use WP_Filesystem::abspath() or WP_Filesystem::wp_*_dir() instead.
 	 * @see WP_Filesystem::abspath()
@@ -131,7 +128,7 @@ class WP_Filesystem_Base {
 	 * @return string The location of the remote path.
 	 */
 	public function find_base_dir( $base = '.', $echo = false ) {
-		_deprecated_function(__FUNCTION__, '2.7', 'WP_Filesystem::abspath() or WP_Filesystem::wp_*_dir()' );
+		_deprecated_function(__FUNCTION__, '2.7.0', 'WP_Filesystem::abspath() or WP_Filesystem::wp_*_dir()' );
 		$this->verbose = $echo;
 		return $this->abspath();
 	}
@@ -139,7 +136,6 @@ class WP_Filesystem_Base {
 	/**
 	 * Locate a folder on the remote filesystem.
 	 *
-	 * @access public
 	 * @since 2.5.0
 	 * @deprecated 2.7.0 use WP_Filesystem::abspath() or WP_Filesystem::wp_*_dir() methods instead.
 	 * @see WP_Filesystem::abspath()
@@ -153,7 +149,7 @@ class WP_Filesystem_Base {
 	 * @return string The location of the remote path.
 	 */
 	public function get_base_dir( $base = '.', $echo = false ) {
-		_deprecated_function(__FUNCTION__, '2.7', 'WP_Filesystem::abspath() or WP_Filesystem::wp_*_dir()' );
+		_deprecated_function(__FUNCTION__, '2.7.0', 'WP_Filesystem::abspath() or WP_Filesystem::wp_*_dir()' );
 		$this->verbose = $echo;
 		return $this->abspath();
 	}
@@ -164,7 +160,6 @@ class WP_Filesystem_Base {
 	 * Assumes that on Windows systems, Stripping off the Drive
 	 * letter is OK Sanitizes \\ to / in windows filepaths.
 	 *
-	 * @access public
 	 * @since 2.7.0
 	 *
 	 * @param string $folder the folder to locate.
@@ -243,8 +238,10 @@ class WP_Filesystem_Base {
 
 		$folder = untrailingslashit($folder);
 
-		if ( $this->verbose )
-			printf( "\n" . __('Looking for %1$s in %2$s') . "<br/>\n", $folder, $base );
+		if ( $this->verbose ) {
+			/* translators: 1: folder to locate, 2: folder to start searching from */
+			printf( "\n" . __( 'Looking for %1$s in %2$s' ) . "<br/>\n", $folder, $base );
+		}
 
 		$folder_parts = explode('/', $folder);
 		$folder_part_keys = array_keys( $folder_parts );
@@ -268,8 +265,10 @@ class WP_Filesystem_Base {
 
 				// Let's try that folder:
 				$newdir = trailingslashit(path_join($base, $key));
-				if ( $this->verbose )
-					printf( "\n" . __('Changing to %s') . "<br/>\n", $newdir );
+				if ( $this->verbose ) {
+					/* translators: %s: directory name */
+					printf( "\n" . __( 'Changing to %s' ) . "<br/>\n", $newdir );
+				}
 
 				// Only search for the remaining path tokens in the directory, not the full path again.
 				$newfolder = implode( '/', array_slice( $folder_parts, $index + 1 ) );
@@ -278,10 +277,13 @@ class WP_Filesystem_Base {
 			}
 		}
 
-		// Only check this as a last resort, to prevent locating the incorrect install. All above procedures will fail quickly if this is the right branch to take.
+		// Only check this as a last resort, to prevent locating the incorrect install.
+		// All above procedures will fail quickly if this is the right branch to take.
 		if (isset( $files[ $last_path ] ) ) {
-			if ( $this->verbose )
-				printf( "\n" . __('Found %s') . "<br/>\n",  $base . $last_path );
+			if ( $this->verbose ) {
+				/* translators: %s: directory name */
+				printf( "\n" . __( 'Found %s' ) . "<br/>\n",  $base . $last_path );
+			}
 			return trailingslashit($base . $last_path);
 		}
 
@@ -301,9 +303,8 @@ class WP_Filesystem_Base {
 	 *
 	 * From the PHP documentation page for fileperms().
 	 *
-	 * @link http://docs.php.net/fileperms
+	 * @link https://secure.php.net/manual/en/function.fileperms.php
 	 *
-	 * @access public
 	 * @since 2.5.0
 	 *
 	 * @param string $file String filename.
@@ -368,9 +369,8 @@ class WP_Filesystem_Base {
 	 * Converts '-rw-r--r--' to 0644
 	 * From "info at rvgate dot nl"'s comment on the PHP documentation for chmod()
  	 *
-	 * @link http://docs.php.net/manual/en/function.chmod.php#49614
+	 * @link https://secure.php.net/manual/en/function.chmod.php#49614
 	 *
-	 * @access public
 	 * @since 2.5.0
 	 *
 	 * @param string $mode string The *nix-style file permission.
@@ -431,6 +431,7 @@ class WP_Filesystem_Base {
 	 *
 	 * @since 2.5.0
 	 * @abstract
+	 *
 	 * @return bool True on success or false on failure (always true for WP_Filesystem_Direct).
 	 */
 	public function connect() {
@@ -442,6 +443,7 @@ class WP_Filesystem_Base {
 	 *
 	 * @since 2.5.0
 	 * @abstract
+	 *
 	 * @param string $file Name of the file to read.
 	 * @return mixed|bool Returns the read data or false on failure.
 	 */
@@ -454,6 +456,7 @@ class WP_Filesystem_Base {
 	 *
 	 * @since 2.5.0
 	 * @abstract
+	 *
 	 * @param string $file Path to the file.
 	 * @return array|bool the file contents in an array or false on failure.
 	 */
@@ -466,6 +469,7 @@ class WP_Filesystem_Base {
 	 *
 	 * @since 2.5.0
 	 * @abstract
+	 *
 	 * @param string $file     Remote path to the file where to write the data.
 	 * @param string $contents The data to write.
 	 * @param int    $mode     Optional. The file permissions as octal number, usually 0644.
@@ -480,6 +484,7 @@ class WP_Filesystem_Base {
 	 *
 	 * @since 2.5.0
 	 * @abstract
+	 *
 	 * @return string|bool The current working directory on success, or false on failure.
 	 */
 	public function cwd() {
@@ -491,6 +496,7 @@ class WP_Filesystem_Base {
 	 *
 	 * @since 2.5.0
 	 * @abstract
+	 *
 	 * @param string $dir The new current directory.
 	 * @return bool|string
 	 */
@@ -503,6 +509,7 @@ class WP_Filesystem_Base {
 	 *
 	 * @since 2.5.0
 	 * @abstract
+	 *
 	 * @param string $file      Path to the file.
 	 * @param mixed  $group     A group name or number.
 	 * @param bool   $recursive Optional. If set True changes file group recursively. Defaults to False.
@@ -517,6 +524,7 @@ class WP_Filesystem_Base {
 	 *
 	 * @since 2.5.0
 	 * @abstract
+	 *
 	 * @param string $file      Path to the file.
 	 * @param int    $mode      Optional. The permissions as octal number, usually 0644 for files, 0755 for dirs.
 	 * @param bool   $recursive Optional. If set True changes file group recursively. Defaults to False.
@@ -531,6 +539,7 @@ class WP_Filesystem_Base {
 	 *
 	 * @since 2.5.0
 	 * @abstract
+	 * 
 	 * @param string $file Path to the file.
 	 * @return string|bool Username of the user or false on error.
 	 */
@@ -543,6 +552,7 @@ class WP_Filesystem_Base {
 	 *
 	 * @since 2.5.0
 	 * @abstract
+	 *
 	 * @param string $file Path to the file.
 	 * @return string|bool The group or false on error.
 	 */
@@ -555,6 +565,7 @@ class WP_Filesystem_Base {
 	 *
 	 * @since 2.5.0
 	 * @abstract
+	 *
 	 * @param string $source      Path to the source file.
 	 * @param string $destination Path to the destination file.
 	 * @param bool   $overwrite   Optional. Whether to overwrite the destination file if it exists.
@@ -572,6 +583,7 @@ class WP_Filesystem_Base {
 	 *
 	 * @since 2.5.0
 	 * @abstract
+	 *
 	 * @param string $source      Path to the source file.
 	 * @param string $destination Path to the destination file.
 	 * @param bool   $overwrite   Optional. Whether to overwrite the destination file if it exists.
@@ -587,6 +599,7 @@ class WP_Filesystem_Base {
 	 *
 	 * @since 2.5.0
 	 * @abstract
+	 *
 	 * @param string $file      Path to the file.
 	 * @param bool   $recursive Optional. If set True changes file group recursively. Defaults to False.
 	 *                          Default false.
@@ -603,6 +616,7 @@ class WP_Filesystem_Base {
 	 *
 	 * @since 2.5.0
 	 * @abstract
+	 *
 	 * @param string $file Path to file/directory.
 	 * @return bool Whether $file exists or not.
 	 */
@@ -615,6 +629,7 @@ class WP_Filesystem_Base {
 	 *
 	 * @since 2.5.0
 	 * @abstract
+	 *
 	 * @param string $file File path.
 	 * @return bool Whether $file is a file.
 	 */
@@ -627,6 +642,7 @@ class WP_Filesystem_Base {
 	 *
 	 * @since 2.5.0
 	 * @abstract
+	 *
 	 * @param string $path Directory path.
 	 * @return bool Whether $path is a directory.
 	 */
@@ -639,6 +655,7 @@ class WP_Filesystem_Base {
 	 *
 	 * @since 2.5.0
 	 * @abstract
+	 *
 	 * @param string $file Path to file.
 	 * @return bool Whether $file is readable.
 	 */
@@ -651,6 +668,8 @@ class WP_Filesystem_Base {
 	 *
 	 * @since 2.5.0
 	 * @abstract
+	 *
+	 * @param string $file Path to file.
 	 * @return bool Whether $file is writable.
 	 */
 	public function is_writable( $file ) {
@@ -662,6 +681,7 @@ class WP_Filesystem_Base {
 	 *
 	 * @since 2.5.0
 	 * @abstract
+	 *
 	 * @param string $file Path to file.
 	 * @return int|bool Unix timestamp representing last access time.
 	 */
@@ -674,6 +694,7 @@ class WP_Filesystem_Base {
 	 *
 	 * @since 2.5.0
 	 * @abstract
+	 *
 	 * @param string $file Path to file.
 	 * @return int|bool Unix timestamp representing modification time.
 	 */
@@ -686,6 +707,7 @@ class WP_Filesystem_Base {
 	 *
 	 * @since 2.5.0
 	 * @abstract
+	 *
 	 * @param string $file Path to file.
 	 * @return int|bool Size of the file in bytes.
 	 */
@@ -700,6 +722,7 @@ class WP_Filesystem_Base {
 	 *
 	 * @since 2.5.0
 	 * @abstract
+	 *
 	 * @param string $file  Path to file.
 	 * @param int    $time  Optional. Modified time to set for file.
 	 *                      Default 0.
@@ -716,6 +739,7 @@ class WP_Filesystem_Base {
 	 *
 	 * @since 2.5.0
 	 * @abstract
+	 *
 	 * @param string $path  Path for new directory.
 	 * @param mixed  $chmod Optional. The permissions as octal number, (or False to skip chmod)
 	 *                      Default false.
@@ -734,6 +758,7 @@ class WP_Filesystem_Base {
 	 *
 	 * @since 2.5.0
 	 * @abstract
+	 *
 	 * @param string $path      Path to directory.
 	 * @param bool   $recursive Optional. Whether to recursively remove files/directories.
 	 *                          Default false.
