@@ -1,13 +1,21 @@
 <?php
 /**
- * Customize Section Class.
- *
- * A UI container for controls, managed by the WP_Customize_Manager.
+ * WordPress Customize Section classes
  *
  * @package WordPress
  * @subpackage Customize
  * @since 3.4.0
  */
+
+/**
+ * Customize Section class.
+ *
+ * A UI container for controls, managed by the WP_Customize_Manager class.
+ *
+ * @since 3.4.0
+ *
+ * @see WP_Customize_Manager
+ */
 class WP_Customize_Section {
 
 	/**
@@ -16,7 +24,8 @@ class WP_Customize_Section {
 	 * Used when sorting two instances whose priorities are equal.
 	 *
 	 * @since 4.1.0
-	 * @access protected
+	 *
+	 * @static
 	 * @var int
 	 */
 	protected static $instance_count = 0;
@@ -25,7 +34,6 @@ class WP_Customize_Section {
 	 * Order in which this instance was created in relation to other instances.
 	 *
 	 * @since 4.1.0
-	 * @access public
 	 * @var int
 	 */
 	public $instance_number;
@@ -34,7 +42,6 @@ class WP_Customize_Section {
 	 * WP_Customize_Manager instance.
 	 *
 	 * @since 3.4.0
-	 * @access public
 	 * @var WP_Customize_Manager
 	 */
 	public $manager;
@@ -43,7 +50,6 @@ class WP_Customize_Section {
 	 * Unique identifier.
 	 *
 	 * @since 3.4.0
-	 * @access public
 	 * @var string
 	 */
 	public $id;
@@ -52,7 +58,6 @@ class WP_Customize_Section {
 	 * Priority of the section which informs load order of sections.
 	 *
 	 * @since 3.4.0
-	 * @access public
 	 * @var integer
 	 */
 	public $priority = 160;
@@ -61,7 +66,6 @@ class WP_Customize_Section {
 	 * Panel in which to show the section, making it a sub-section.
 	 *
 	 * @since 4.0.0
-	 * @access public
 	 * @var string
 	 */
 	public $panel = '';
@@ -70,7 +74,6 @@ class WP_Customize_Section {
 	 * Capability required for the section.
 	 *
 	 * @since 3.4.0
-	 * @access public
 	 * @var string
 	 */
 	public $capability = 'edit_theme_options';
@@ -79,7 +82,6 @@ class WP_Customize_Section {
 	 * Theme feature support for the section.
 	 *
 	 * @since 3.4.0
-	 * @access public
 	 * @var string|array
 	 */
 	public $theme_supports = '';
@@ -88,7 +90,6 @@ class WP_Customize_Section {
 	 * Title of the section to show in UI.
 	 *
 	 * @since 3.4.0
-	 * @access public
 	 * @var string
 	 */
 	public $title = '';
@@ -97,7 +98,6 @@ class WP_Customize_Section {
 	 * Description to show in the UI.
 	 *
 	 * @since 3.4.0
-	 * @access public
 	 * @var string
 	 */
 	public $description = '';
@@ -106,7 +106,6 @@ class WP_Customize_Section {
 	 * Customizer controls for this section.
 	 *
 	 * @since 3.4.0
-	 * @access public
 	 * @var array
 	 */
 	public $controls;
@@ -115,7 +114,6 @@ class WP_Customize_Section {
 	 * Type of this section.
 	 *
 	 * @since 4.1.0
-	 * @access public
 	 * @var string
 	 */
 	public $type = 'default';
@@ -124,17 +122,27 @@ class WP_Customize_Section {
 	 * Active callback.
 	 *
 	 * @since 4.1.0
-	 * @access public
 	 *
 	 * @see WP_Customize_Section::active()
 	 *
 	 * @var callable Callback is called with one argument, the instance of
-	 *               {@see WP_Customize_Section}, and returns bool to indicate
-	 *               whether the section is active (such as it relates to the URL
-	 *               currently being previewed).
+	 *               WP_Customize_Section, and returns bool to indicate whether
+	 *               the section is active (such as it relates to the URL currently
+	 *               being previewed).
 	 */
 	public $active_callback = '';
 
+	/**
+	 * Show the description or hide it behind the help icon.
+	 *
+	 * @since 4.7.0
+	 *
+	 * @var bool Indicates whether the Section's description should be
+	 *           hidden behind a help icon ("?") in the Section header,
+	 *           similar to how help icons are displayed on Panels.
+	 */
+	public $description_hidden = false;
+
 	/**
 	 * Constructor.
 	 *
@@ -163,29 +171,26 @@ class WP_Customize_Section {
 		$this->instance_number = self::$instance_count;
 
 		$this->controls = array(); // Users cannot customize the $controls array.
-
-		return $this;
 	}
 
 	/**
 	 * Check whether section is active to current Customizer preview.
 	 *
 	 * @since 4.1.0
-	 * @access public
 	 *
 	 * @return bool Whether the section is active to the current preview.
 	 */
-	public final function active() {
+	final public function active() {
 		$section = $this;
 		$active = call_user_func( $this->active_callback, $this );
 
 		/**
-		 * Filter response of {@see WP_Customize_Section::active()}.
+		 * Filters response of WP_Customize_Section::active().
 		 *
 		 * @since 4.1.0
 		 *
 		 * @param bool                 $active  Whether the Customizer section is active.
-		 * @param WP_Customize_Section $section {@see WP_Customize_Section} instance.
+		 * @param WP_Customize_Section $section WP_Customize_Section instance.
 		 */
 		$active = apply_filters( 'customize_section_active', $active, $section );
 
@@ -193,15 +198,14 @@ class WP_Customize_Section {
 	}
 
 	/**
-	 * Default callback used when invoking {@see WP_Customize_Section::active()}.
+	 * Default callback used when invoking WP_Customize_Section::active().
 	 *
 	 * Subclasses can override this with their specific logic, or they may provide
 	 * an 'active_callback' argument to the constructor.
 	 *
 	 * @since 4.1.0
-	 * @access public
 	 *
-	 * @return bool Always true.
+	 * @return true Always true.
 	 */
 	public function active_callback() {
 		return true;
@@ -215,10 +219,19 @@ class WP_Customize_Section {
 	 * @return array The array to be exported to the client as JSON.
 	 */
 	public function json() {
-		$array = wp_array_slice_assoc( (array) $this, array( 'title', 'description', 'priority', 'panel', 'type' ) );
+		$array = wp_array_slice_assoc( (array) $this, array( 'id', 'description', 'priority', 'panel', 'type', 'description_hidden' ) );
+		$array['title'] = html_entity_decode( $this->title, ENT_QUOTES, get_bloginfo( 'charset' ) );
 		$array['content'] = $this->get_content();
 		$array['active'] = $this->active();
 		$array['instanceNumber'] = $this->instance_number;
+
+		if ( $this->panel ) {
+			/* translators: &#9656; is the unicode right-pointing triangle, and %s is the section title in the Customizer */
+			$array['customizeAction'] = sprintf( __( 'Customizing &#9656; %s' ), esc_html( $this->manager->get_panel( $this->panel )->title ) );
+		} else {
+			$array['customizeAction'] = __( 'Customizing' );
+		}
+
 		return $array;
 	}
 
@@ -230,7 +243,7 @@ class WP_Customize_Section {
 	 *
 	 * @return bool False if theme doesn't support the section or user doesn't have the capability.
 	 */
-	public final function check_capabilities() {
+	final public function check_capabilities() {
 		if ( $this->capability && ! call_user_func_array( 'current_user_can', (array) $this->capability ) ) {
 			return false;
 		}
@@ -243,18 +256,16 @@ class WP_Customize_Section {
 	}
 
 	/**
-	 * Get the section's content template for insertion into the Customizer pane.
+	 * Get the section's content for insertion into the Customizer pane.
 	 *
 	 * @since 4.1.0
 	 *
 	 * @return string Contents of the section.
 	 */
-	public final function get_content() {
+	final public function get_content() {
 		ob_start();
 		$this->maybe_render();
-		$template = trim( ob_get_contents() );
-		ob_end_clean();
-		return $template;
+		return trim( ob_get_clean() );
 	}
 
 	/**
@@ -262,7 +273,7 @@ class WP_Customize_Section {
 	 *
 	 * @since 3.4.0
 	 */
-	public final function maybe_render() {
+	final public function maybe_render() {
 		if ( ! $this->check_capabilities() ) {
 			return;
 		}
@@ -289,82 +300,99 @@ class WP_Customize_Section {
 	}
 
 	/**
-	 * Render the section, and the controls that have been added to it.
+	 * Render the section UI in a subclass.
+	 *
+	 * Sections are now rendered in JS by default, see WP_Customize_Section::print_template().
 	 *
 	 * @since 3.4.0
 	 */
-	protected function render() {
-		$classes = 'accordion-section control-section control-section-' . $this->type;
+	protected function render() {}
+
+	/**
+	 * Render the section's JS template.
+	 *
+	 * This function is only run for section types that have been registered with
+	 * WP_Customize_Manager::register_section_type().
+	 *
+	 * @since 4.3.0
+	 *
+	 * @see WP_Customize_Manager::render_template()
+	 */
+	public function print_template() {
 		?>
-		<li id="accordion-section-<?php echo esc_attr( $this->id ); ?>" class="<?php echo esc_attr( $classes ); ?>">
+		<script type="text/html" id="tmpl-customize-section-<?php echo $this->type; ?>">
+			<?php $this->render_template(); ?>
+		</script>
+		<?php
+	}
+
+	/**
+	 * An Underscore (JS) template for rendering this section.
+	 *
+	 * Class variables for this section class are available in the `data` JS object;
+	 * export custom variables by overriding WP_Customize_Section::json().
+	 *
+	 * @since 4.3.0
+	 *
+	 * @see WP_Customize_Section::print_template()
+	 */
+	protected function render_template() {
+		?>
+		<li id="accordion-section-{{ data.id }}" class="accordion-section control-section control-section-{{ data.type }}">
 			<h3 class="accordion-section-title" tabindex="0">
-				<?php echo esc_html( $this->title ); ?>
-				<span class="screen-reader-text"><?php _e( 'Press return or enter to expand' ); ?></span>
+				{{ data.title }}
+				<span class="screen-reader-text"><?php _e( 'Press return or enter to open this section' ); ?></span>
 			</h3>
 			<ul class="accordion-section-content">
-				<?php if ( ! empty( $this->description ) ) : ?>
-					<li class="customize-section-description-container">
-						<p class="description customize-section-description"><?php echo $this->description; ?></p>
-					</li>
-				<?php endif; ?>
+				<li class="customize-section-description-container section-meta <# if ( data.description_hidden ) { #>customize-info<# } #>">
+					<div class="customize-section-title">
+						<button class="customize-section-back" tabindex="-1">
+							<span class="screen-reader-text"><?php _e( 'Back' ); ?></span>
+						</button>
+						<h3>
+							<span class="customize-action">
+								{{{ data.customizeAction }}}
+							</span>
+							{{ data.title }}
+						</h3>
+						<# if ( data.description && data.description_hidden ) { #>
+							<button type="button" class="customize-help-toggle dashicons dashicons-editor-help" aria-expanded="false"><span class="screen-reader-text"><?php _e( 'Help' ); ?></span></button>
+							<div class="description customize-section-description">
+								{{{ data.description }}}
+							</div>
+						<# } #>
+
+						<div class="customize-control-notifications-container"></div>
+					</div>
+
+					<# if ( data.description && ! data.description_hidden ) { #>
+						<div class="description customize-section-description">
+							{{{ data.description }}}
+						</div>
+					<# } #>
+				</li>
 			</ul>
 		</li>
 		<?php
 	}
 }
 
+/** WP_Customize_Themes_Section class */
+require_once( ABSPATH . WPINC . '/customize/class-wp-customize-themes-section.php' );
+
+/** WP_Customize_Sidebar_Section class */
+require_once( ABSPATH . WPINC . '/customize/class-wp-customize-sidebar-section.php' );
+
+/** WP_Customize_Nav_Menu_Section class */
+require_once( ABSPATH . WPINC . '/customize/class-wp-customize-nav-menu-section.php' );
+
 /**
- * Customizer section representing widget area (sidebar).
+ * WP_Customize_New_Menu_Section class
  *
- * @package WordPress
- * @subpackage Customize
+ * As this file is deprecated, it will trigger a deprecation notice if instantiated. In a subsequent
+ * release, the require_once() here will be removed and _deprecated_file() will be called if file is
+ * required at all.
  *
- * @since 4.1.0
- *
- * @see WP_Customize_Section
+ * @deprecated 4.9.0 This file is no longer used due to new menu creation UX.
  */
-class WP_Customize_Sidebar_Section extends WP_Customize_Section {
-
-	/**
-	 * Type of this section.
-	 *
-	 * @since 4.1.0
-	 * @access public
-	 * @var string
-	 */
-	public $type = 'sidebar';
-
-	/**
-	 * Unique identifier.
-	 *
-	 * @since 4.1.0
-	 * @access public
-	 * @var string
-	 */
-	public $sidebar_id;
-
-	/**
-	 * Gather the parameters passed to client JavaScript via JSON.
-	 *
-	 * @since 4.1.0
-	 *
-	 * @return array The array to be exported to the client as JSON.
-	 */
-	public function json() {
-		$json = parent::json();
-		$json['sidebarId'] = $this->sidebar_id;
-		return $json;
-	}
-
-	/**
-	 * Whether the current sidebar is rendered on the page.
-	 *
-	 * @since 4.1.0
-	 * @access public
-	 *
-	 * @return bool Whether sidebar is rendered.
-	 */
-	public function active_callback() {
-		return $this->manager->widgets->is_sidebar_rendered( $this->sidebar_id );
-	}
-}
+require_once( ABSPATH . WPINC . '/customize/class-wp-customize-new-menu-section.php' );
