@@ -12,7 +12,6 @@ final class WP_Theme implements ArrayAccess {
 	 * Whether the theme has been marked as updateable.
 	 *
 	 * @since 4.4.0
-	 * @access public
 	 * @var bool
 	 *
 	 * @see WP_MS_Themes_List_Table
@@ -23,7 +22,6 @@ final class WP_Theme implements ArrayAccess {
 	 * Headers for style.css files.
 	 *
 	 * @static
-	 * @access private
 	 * @var array
 	 */
 	private static $file_headers = array(
@@ -44,26 +42,25 @@ final class WP_Theme implements ArrayAccess {
 	 * Default themes.
 	 *
 	 * @static
-	 * @access private
 	 * @var array
 	 */
 	private static $default_themes = array(
-		'classic'        => 'WordPress Classic',
-		'default'        => 'WordPress Default',
-		'twentyten'      => 'Twenty Ten',
-		'twentyeleven'   => 'Twenty Eleven',
-		'twentytwelve'   => 'Twenty Twelve',
-		'twentythirteen' => 'Twenty Thirteen',
-		'twentyfourteen' => 'Twenty Fourteen',
-		'twentyfifteen'  => 'Twenty Fifteen',
-		'twentysixteen'  => 'Twenty Sixteen',
+		'classic'         => 'WordPress Classic',
+		'default'         => 'WordPress Default',
+		'twentyten'       => 'Twenty Ten',
+		'twentyeleven'    => 'Twenty Eleven',
+		'twentytwelve'    => 'Twenty Twelve',
+		'twentythirteen'  => 'Twenty Thirteen',
+		'twentyfourteen'  => 'Twenty Fourteen',
+		'twentyfifteen'   => 'Twenty Fifteen',
+		'twentysixteen'   => 'Twenty Sixteen',
+		'twentyseventeen' => 'Twenty Seventeen',
 	);
 
 	/**
 	 * Renamed theme tags.
 	 *
 	 * @static
-	 * @access private
 	 * @var array
 	 */
 	private static $tag_map = array(
@@ -74,7 +71,6 @@ final class WP_Theme implements ArrayAccess {
 	/**
 	 * Absolute path to the theme root, usually wp-content/themes
 	 *
-	 * @access private
 	 * @var string
 	 */
 	private $theme_root;
@@ -82,7 +78,6 @@ final class WP_Theme implements ArrayAccess {
 	/**
 	 * Header data from the theme's style.css file.
 	 *
-	 * @access private
 	 * @var array
 	 */
 	private $headers = array();
@@ -90,7 +85,6 @@ final class WP_Theme implements ArrayAccess {
 	/**
 	 * Header data from the theme's style.css file after being sanitized.
 	 *
-	 * @access private
 	 * @var array
 	 */
 	private $headers_sanitized;
@@ -100,7 +94,6 @@ final class WP_Theme implements ArrayAccess {
 	 *
 	 * Cached due to sorting functions running over the translated name.
 	 *
-	 * @access private
 	 * @var string
 	 */
 	private $name_translated;
@@ -108,7 +101,6 @@ final class WP_Theme implements ArrayAccess {
 	/**
 	 * Errors encountered when initializing the theme.
 	 *
-	 * @access private
 	 * @var WP_Error
 	 */
 	private $errors;
@@ -119,7 +111,6 @@ final class WP_Theme implements ArrayAccess {
 	 * In the case of a child theme, this is directory name of the child theme.
 	 * Otherwise, 'stylesheet' is the same as 'template'.
 	 *
-	 * @access private
 	 * @var string
 	 */
 	private $stylesheet;
@@ -130,7 +121,6 @@ final class WP_Theme implements ArrayAccess {
 	 * In the case of a child theme, this is the directory name of the parent theme.
 	 * Otherwise, 'template' is the same as 'stylesheet'.
 	 *
-	 * @access private
 	 * @var string
 	 */
 	private $template;
@@ -138,7 +128,6 @@ final class WP_Theme implements ArrayAccess {
 	/**
 	 * A reference to the parent theme, in the case of a child theme.
 	 *
-	 * @access private
 	 * @var WP_Theme
 	 */
 	private $parent;
@@ -146,15 +135,13 @@ final class WP_Theme implements ArrayAccess {
 	/**
 	 * URL to the theme root, usually an absolute URL to wp-content/themes
 	 *
-	 * @access private
-	 * var string
+	 * @var string
 	 */
 	private $theme_root_uri;
 
 	/**
 	 * Flag for whether the theme's textdomain is loaded.
 	 *
-	 * @access private
 	 * @var bool
 	 */
 	private $textdomain_loaded;
@@ -162,7 +149,6 @@ final class WP_Theme implements ArrayAccess {
 	/**
 	 * Stores an md5 hash of the theme root, to function as the cache key.
 	 *
-	 * @access private
 	 * @var string
 	 */
 	private $cache_hash;
@@ -170,10 +156,9 @@ final class WP_Theme implements ArrayAccess {
 	/**
 	 * Flag for whether the themes cache bucket should be persistently cached.
 	 *
-	 * Default is false. Can be set with the wp_cache_themes_persistently filter.
+	 * Default is false. Can be set with the {@see 'wp_cache_themes_persistently'} filter.
 	 *
 	 * @static
-	 * @access private
 	 * @var bool
 	 */
 	private static $persistently_cache;
@@ -184,7 +169,6 @@ final class WP_Theme implements ArrayAccess {
 	 * By default the bucket is not cached, so this value is useless.
 	 *
 	 * @static
-	 * @access private
 	 * @var bool
 	 */
 	private static $cache_expiration = 1800;
@@ -192,6 +176,8 @@ final class WP_Theme implements ArrayAccess {
 	/**
 	 * Constructor for WP_Theme.
 	 *
+	 * @since  3.4.0
+	 *
 	 * @global array $wp_theme_directories
 	 *
 	 * @param string $theme_dir Directory of the theme within the theme_root.
@@ -264,11 +250,26 @@ final class WP_Theme implements ArrayAccess {
 			}
 		}
 
+		if ( ! $this->template && $this->stylesheet === $this->headers['Template'] ) {
+			/* translators: %s: Template */
+			$this->errors = new WP_Error( 'theme_child_invalid', sprintf( __( 'The theme defines itself as its parent theme. Please check the %s header.' ), '<code>Template</code>' ) );
+			$this->cache_add( 'theme', array( 'headers' => $this->headers, 'errors' => $this->errors, 'stylesheet' => $this->stylesheet ) );
+
+			return;
+		}
+
 		// (If template is set from cache [and there are no errors], we know it's good.)
 		if ( ! $this->template && ! ( $this->template = $this->headers['Template'] ) ) {
 			$this->template = $this->stylesheet;
 			if ( ! file_exists( $this->theme_root . '/' . $this->stylesheet . '/index.php' ) ) {
-				$this->errors = new WP_Error( 'theme_no_index', __( 'Template is missing.' ) );
+				$error_message = sprintf(
+					/* translators: 1: index.php, 2: Codex URL, 3: style.css */
+					__( 'Template is missing. Standalone themes need to have a %1$s template file. <a href="%2$s">Child themes</a> need to have a Template header in the %3$s stylesheet.' ),
+					'<code>index.php</code>',
+					__( 'https://codex.wordpress.org/Child_Themes' ),
+					'<code>style.css</code>'
+				);
+				$this->errors = new WP_Error( 'theme_no_index', $error_message );
 				$this->cache_add( 'theme', array( 'headers' => $this->headers, 'errors' => $this->errors, 'stylesheet' => $this->stylesheet, 'template' => $this->template ) );
 				return;
 			}
@@ -325,6 +326,8 @@ final class WP_Theme implements ArrayAccess {
 	/**
 	 * When converting the object to a string, the theme name is returned.
 	 *
+	 * @since  3.4.0
+	 *
 	 * @return string Theme name, ready for display (translated)
 	 */
 	public function __toString() {
@@ -336,7 +339,10 @@ final class WP_Theme implements ArrayAccess {
 	 *
 	 * @staticvar array $properties
 	 *
-	 * @return bool
+	 * @since  3.4.0
+	 *
+	 * @param string $offset Property to check if set.
+	 * @return bool Whether the given property is set.
 	 */
 	public function __isset( $offset ) {
 		static $properties = array(
@@ -350,7 +356,10 @@ final class WP_Theme implements ArrayAccess {
 	/**
 	 * __get() magic method for properties formerly returned by current_theme_info()
 	 *
-	 * @return mixed
+	 * @since  3.4.0
+	 *
+	 * @param string $offset Property to get.
+	 * @return mixed Property value.
 	 */
 	public function __get( $offset ) {
 		switch ( $offset ) {
@@ -391,6 +400,8 @@ final class WP_Theme implements ArrayAccess {
 	/**
 	 * Method to implement ArrayAccess for keys formerly returned by get_themes()
 	 *
+	 * @since  3.4.0
+	 *
 	 * @param mixed $offset
 	 * @param mixed $value
 	 */
@@ -399,6 +410,8 @@ final class WP_Theme implements ArrayAccess {
 	/**
 	 * Method to implement ArrayAccess for keys formerly returned by get_themes()
 	 *
+	 * @since  3.4.0
+	 *
 	 * @param mixed $offset
 	 */
 	public function offsetUnset( $offset ) {}
@@ -408,6 +421,8 @@ final class WP_Theme implements ArrayAccess {
 	 *
 	 * @staticvar array $keys
 	 *
+	 * @since  3.4.0
+	 *
 	 * @param mixed $offset
 	 * @return bool
 	 */
@@ -428,9 +443,11 @@ final class WP_Theme implements ArrayAccess {
 	 * translated data. We are doing so now as it is safe to do. However, as
 	 * Name and Title could have been used as the key for get_themes(), both remain
 	 * untranslated for back compatibility. This means that ['Name'] is not ideal,
-	 * and care should be taken to use $theme->display('Name') to get a properly
+	 * and care should be taken to use `$theme::display( 'Name' )` to get a properly
 	 * translated header.
 	 *
+	 * @since  3.4.0
+	 *
 	 * @param mixed $offset
 	 * @return mixed
 	 */
@@ -438,8 +455,10 @@ final class WP_Theme implements ArrayAccess {
 		switch ( $offset ) {
 			case 'Name' :
 			case 'Title' :
-				// See note above about using translated data. get() is not ideal.
-				// It is only for backwards compatibility. Use display().
+				/*
+				 * See note above about using translated data. get() is not ideal.
+				 * It is only for backward compatibility. Use display().
+				 */
 				return $this->get('Name');
 			case 'Author' :
 				return $this->display( 'Author');
@@ -483,7 +502,6 @@ final class WP_Theme implements ArrayAccess {
 	 * Returns errors property.
 	 *
 	 * @since 3.4.0
-	 * @access public
 	 *
 	 * @return WP_Error|false WP_Error if there are errors, or false.
 	 */
@@ -498,7 +516,6 @@ final class WP_Theme implements ArrayAccess {
 	 * meaning that the theme's directory was not found, does not exist.
 	 *
 	 * @since 3.4.0
-	 * @access public
 	 *
 	 * @return bool Whether the theme exists.
 	 */
@@ -510,7 +527,6 @@ final class WP_Theme implements ArrayAccess {
 	 * Returns reference to the parent theme.
 	 *
 	 * @since 3.4.0
-	 * @access public
 	 *
 	 * @return WP_Theme|false Parent theme, or false if the current theme is not a child theme.
 	 */
@@ -524,9 +540,8 @@ final class WP_Theme implements ArrayAccess {
 	 * Cache entries keyed by the theme and the type of data.
 	 *
 	 * @since 3.4.0
-	 * @access private
 	 *
-	 * @param string $key Type of data to store (theme, screenshot, headers, page_templates)
+	 * @param string $key Type of data to store (theme, screenshot, headers, post_templates)
 	 * @param string $data Data to store
 	 * @return bool Return value from wp_cache_add()
 	 */
@@ -540,9 +555,8 @@ final class WP_Theme implements ArrayAccess {
 	 * Cache entries are keyed by the theme and the type of data.
 	 *
 	 * @since 3.4.0
-	 * @access private
 	 *
-	 * @param string $key Type of data to retrieve (theme, screenshot, headers, page_templates)
+	 * @param string $key Type of data to retrieve (theme, screenshot, headers, post_templates)
 	 * @return mixed Retrieved data
 	 */
 	private function cache_get( $key ) {
@@ -553,10 +567,9 @@ final class WP_Theme implements ArrayAccess {
 	 * Clears the cache for the theme.
 	 *
 	 * @since 3.4.0
-	 * @access public
 	 */
 	public function cache_delete() {
-		foreach ( array( 'theme', 'screenshot', 'headers', 'page_templates' ) as $key )
+		foreach ( array( 'theme', 'screenshot', 'headers', 'post_templates' ) as $key )
 			wp_cache_delete( $key . '-' . $this->cache_hash, 'themes' );
 		$this->template = $this->textdomain_loaded = $this->theme_root_uri = $this->parent = $this->errors = $this->headers_sanitized = $this->name_translated = null;
 		$this->headers = array();
@@ -575,7 +588,6 @@ final class WP_Theme implements ArrayAccess {
 	 * whether it is actually valid.
 	 *
 	 * @since 3.4.0
-	 * @access public
 	 *
 	 * @param string $header Theme header. Name, Description, Author, Version, ThemeURI, AuthorURI, Status, Tags.
 	 * @return string|false String on success, false on failure.
@@ -609,7 +621,6 @@ final class WP_Theme implements ArrayAccess {
 	 * Gets a theme header, formatted and translated for display.
 	 *
 	 * @since 3.4.0
-	 * @access public
 	 *
 	 * @param string $header Theme header. Name, Description, Author, Version, ThemeURI, AuthorURI, Status, Tags.
 	 * @param bool $markup Optional. Whether to mark up the header. Defaults to true.
@@ -638,7 +649,6 @@ final class WP_Theme implements ArrayAccess {
 	 * Sanitize a theme header.
 	 *
 	 * @since 3.4.0
-	 * @access private
 	 *
 	 * @staticvar array $header_tags
 	 * @staticvar array $header_tags_with_a
@@ -697,7 +707,6 @@ final class WP_Theme implements ArrayAccess {
 	 * Mark up a theme header.
 	 *
      * @since 3.4.0
-	 * @access private
 	 *
 	 * @staticvar string $comma
 	 *
@@ -709,8 +718,9 @@ final class WP_Theme implements ArrayAccess {
 	private function markup_header( $header, $value, $translate ) {
 		switch ( $header ) {
 			case 'Name' :
-				if ( empty( $value ) )
-					$value = $this->get_stylesheet();
+				if ( empty( $value ) ) {
+					$value = esc_html( $this->get_stylesheet() );
+				}
 				break;
 			case 'Description' :
 				$value = wptexturize( $value );
@@ -743,7 +753,6 @@ final class WP_Theme implements ArrayAccess {
 	 * Translate a theme header.
 	 *
 	 * @since 3.4.0
-	 * @access private
 	 *
 	 * @staticvar array $tags_list
 	 *
@@ -760,15 +769,28 @@ final class WP_Theme implements ArrayAccess {
 				$this->name_translated = translate( $value, $this->get('TextDomain' ) );
 				return $this->name_translated;
 			case 'Tags' :
-				if ( empty( $value ) || ! function_exists( 'get_theme_feature_list' ) )
+				if ( empty( $value ) || ! function_exists( 'get_theme_feature_list' ) ) {
 					return $value;
+				}
 
 				static $tags_list;
 				if ( ! isset( $tags_list ) ) {
-					$tags_list = array();
+					$tags_list = array(
+						// As of 4.6, deprecated tags which are only used to provide translation for older themes.
+						'black' => __( 'Black' ), 'blue' => __( 'Blue' ), 'brown'  => __( 'Brown' ),
+						'gray' => __( 'Gray' ), 'green'  => __( 'Green' ), 'orange' => __( 'Orange' ),
+						'pink' => __( 'Pink' ), 'purple' => __( 'Purple' ), 'red' => __( 'Red' ),
+						'silver' => __( 'Silver' ), 'tan' => __( 'Tan' ), 'white' => __( 'White' ),
+						'yellow' => __( 'Yellow' ), 'dark' => __( 'Dark' ), 'light' => __( 'Light' ),
+						'fixed-layout' => __( 'Fixed Layout' ), 'fluid-layout' => __( 'Fluid Layout' ),
+						'responsive-layout' => __( 'Responsive Layout' ), 'blavatar' => __( 'Blavatar' ),
+						'photoblogging' => __( 'Photoblogging' ), 'seasonal' => __( 'Seasonal' ),
+					);
+
 					$feature_list = get_theme_feature_list( false ); // No API
-					foreach ( $feature_list as $tags )
+					foreach ( $feature_list as $tags ) {
 						$tags_list += $tags;
+					}
 				}
 
 				foreach ( $value as &$tag ) {
@@ -794,7 +816,6 @@ final class WP_Theme implements ArrayAccess {
 	 * Otherwise, get_stylesheet() is the same as get_template().
 	 *
 	 * @since 3.4.0
-	 * @access public
 	 *
 	 * @return string Stylesheet
 	 */
@@ -809,7 +830,6 @@ final class WP_Theme implements ArrayAccess {
 	 * Otherwise, the get_template() is the same as get_stylesheet().
 	 *
 	 * @since 3.4.0
-	 * @access public
 	 *
 	 * @return string Template
 	 */
@@ -824,7 +844,6 @@ final class WP_Theme implements ArrayAccess {
 	 * of the child theme's files.
 	 *
 	 * @since 3.4.0
-	 * @access public
 	 *
 	 * @return string Absolute path of the stylesheet directory.
 	 */
@@ -842,7 +861,6 @@ final class WP_Theme implements ArrayAccess {
 	 * of the parent theme's files.
 	 *
 	 * @since 3.4.0
-	 * @access public
 	 *
 	 * @return string Absolute path of the template directory.
 	 */
@@ -862,7 +880,6 @@ final class WP_Theme implements ArrayAccess {
 	 * child theme's files.
 	 *
 	 * @since 3.4.0
-	 * @access public
 	 *
 	 * @return string URL to the stylesheet directory.
 	 */
@@ -877,7 +894,6 @@ final class WP_Theme implements ArrayAccess {
 	 * parent theme's files.
 	 *
 	 * @since 3.4.0
-	 * @access public
 	 *
 	 * @return string URL to the template directory.
 	 */
@@ -896,7 +912,6 @@ final class WP_Theme implements ArrayAccess {
 	 * This is typically the absolute path to wp-content/themes.
 	 *
 	 * @since 3.4.0
-	 * @access public
 	 *
 	 * @return string Theme root.
 	 */
@@ -909,10 +924,9 @@ final class WP_Theme implements ArrayAccess {
 	 *
 	 * This is typically the absolute URL to wp-content/themes. This forms the basis
 	 * for all other URLs returned by WP_Theme, so we pass it to the public function
-	 * get_theme_root_uri() and allow it to run the theme_root_uri filter.
+	 * get_theme_root_uri() and allow it to run the {@see 'theme_root_uri'} filter.
 	 *
 	 * @since 3.4.0
-	 * @access public
 	 *
 	 * @return string Theme root URI.
 	 */
@@ -931,7 +945,6 @@ final class WP_Theme implements ArrayAccess {
 	 * themes, parent theme screenshots are not inherited.)
 	 *
 	 * @since 3.4.0
-	 * @access public
 	 *
 	 * @param string $uri Type of URL to return, either 'relative' or an absolute URI. Defaults to absolute URI.
 	 * @return string|false Screenshot file. False if the theme does not have a screenshot.
@@ -963,74 +976,113 @@ final class WP_Theme implements ArrayAccess {
 	 * Return files in the theme's directory.
 	 *
 	 * @since 3.4.0
-	 * @access public
 	 *
 	 * @param mixed $type Optional. Array of extensions to return. Defaults to all files (null).
 	 * @param int $depth Optional. How deep to search for files. Defaults to a flat scan (0 depth). -1 depth is infinite.
 	 * @param bool $search_parent Optional. Whether to return parent files. Defaults to false.
 	 * @return array Array of files, keyed by the path to the file relative to the theme's directory, with the values
-	 * 	             being absolute paths.
+	 *               being absolute paths.
 	 */
 	public function get_files( $type = null, $depth = 0, $search_parent = false ) {
 		$files = (array) self::scandir( $this->get_stylesheet_directory(), $type, $depth );
 
-		if ( $search_parent && $this->parent() )
+		if ( $search_parent && $this->parent() ) {
 			$files += (array) self::scandir( $this->get_template_directory(), $type, $depth );
+		}
 
 		return $files;
 	}
 
 	/**
-	 * Returns the theme's page templates.
+	 * Returns the theme's post templates.
 	 *
-	 * @since 3.4.0
-	 * @access public
+	 * @since 4.7.0
 	 *
-	 * @param WP_Post|null $post Optional. The post being edited, provided for context.
-	 * @return array Array of page templates, keyed by filename, with the value of the translated header name.
+	 * @return array Array of page templates, keyed by filename and post type,
+	 *               with the value of the translated header name.
 	 */
-	public function get_page_templates( $post = null ) {
+	public function get_post_templates() {
 		// If you screw up your current theme and we invalidate your parent, most things still work. Let it slide.
-		if ( $this->errors() && $this->errors()->get_error_codes() !== array( 'theme_parent_invalid' ) )
+		if ( $this->errors() && $this->errors()->get_error_codes() !== array( 'theme_parent_invalid' ) ) {
 			return array();
+		}
 
-		$page_templates = $this->cache_get( 'page_templates' );
+		$post_templates = $this->cache_get( 'post_templates' );
 
-		if ( ! is_array( $page_templates ) ) {
-			$page_templates = array();
+		if ( ! is_array( $post_templates ) ) {
+			$post_templates = array();
 
-			$files = (array) $this->get_files( 'php', 1 );
+			$files = (array) $this->get_files( 'php', 1, true);
 
 			foreach ( $files as $file => $full_path ) {
-				if ( ! preg_match( '|Template Name:(.*)$|mi', file_get_contents( $full_path ), $header ) )
+				if ( ! preg_match( '|Template Name:(.*)$|mi', file_get_contents( $full_path ), $header ) ) {
 					continue;
-				$page_templates[ $file ] = _cleanup_header_comment( $header[1] );
+				}
+
+				$types = array( 'page' );
+				if ( preg_match( '|Template Post Type:(.*)$|mi', file_get_contents( $full_path ), $type ) ) {
+					$types = explode( ',', _cleanup_header_comment( $type[1] ) );
+				}
+
+				foreach ( $types as $type ) {
+					$type = sanitize_key( $type );
+					if ( ! isset( $post_templates[ $type ] ) ) {
+						$post_templates[ $type ] = array();
+					}
+
+					$post_templates[ $type ][ $file ] = _cleanup_header_comment( $header[1] );
+				}
 			}
 
-			$this->cache_add( 'page_templates', $page_templates );
+			$this->cache_add( 'post_templates', $post_templates );
 		}
 
 		if ( $this->load_textdomain() ) {
-			foreach ( $page_templates as &$page_template ) {
-				$page_template = $this->translate_header( 'Template Name', $page_template );
+			foreach ( $post_templates as &$post_type ) {
+				foreach ( $post_type as &$post_template ) {
+					$post_template = $this->translate_header( 'Template Name', $post_template );
+				}
 			}
 		}
 
-		if ( $this->parent() )
-			$page_templates += $this->parent()->get_page_templates( $post );
+		return $post_templates;
+	}
+
+	/**
+	 * Returns the theme's post templates for a given post type.
+	 *
+	 * @since 3.4.0
+	 * @since 4.7.0 Added the `$post_type` parameter.
+	 *
+	 * @param WP_Post|null $post      Optional. The post being edited, provided for context.
+	 * @param string       $post_type Optional. Post type to get the templates for. Default 'page'.
+	 *                                If a post is provided, its post type is used.
+	 * @return array Array of page templates, keyed by filename, with the value of the translated header name.
+	 */
+	public function get_page_templates( $post = null, $post_type = 'page' ) {
+		if ( $post ) {
+			$post_type = get_post_type( $post );
+		}
+
+		$post_templates = $this->get_post_templates();
+		$post_templates = isset( $post_templates[ $post_type ] ) ? $post_templates[ $post_type ] : array();
 
 		/**
-		 * Filter list of page templates for a theme.
+		 * Filters list of page templates for a theme.
+		 *
+		 * The dynamic portion of the hook name, `$post_type`, refers to the post type.
 		 *
 		 * @since 3.9.0
 		 * @since 4.4.0 Converted to allow complete control over the `$page_templates` array.
+		 * @since 4.7.0 Added the `$post_type` parameter.
 		 *
-		 * @param array        $page_templates Array of page templates. Keys are filenames,
+		 * @param array        $post_templates Array of page templates. Keys are filenames,
 		 *                                     values are translated names.
 		 * @param WP_Theme     $this           The theme object.
 		 * @param WP_Post|null $post           The post being edited, provided for context, or null.
+		 * @param string       $post_type      Post type to get the templates for.
 		 */
-		return (array) apply_filters( 'theme_page_templates', $page_templates, $this, $post );
+		return (array) apply_filters( "theme_{$post_type}_templates", $post_templates, $this, $post, $post_type );
 	}
 
 	/**
@@ -1039,7 +1091,6 @@ final class WP_Theme implements ArrayAccess {
 	 * @since 3.4.0
 	 *
 	 * @static
-	 * @access private
 	 *
 	 * @param string            $path          Absolute path to search.
 	 * @param array|string|null $extensions    Optional. Array of extensions to find, string of a single extension,
@@ -1053,8 +1104,9 @@ final class WP_Theme implements ArrayAccess {
 	 *                     with `$relative_path`, with the values being absolute paths. False otherwise.
 	 */
 	private static function scandir( $path, $extensions = null, $depth = 0, $relative_path = '' ) {
-		if ( ! is_dir( $path ) )
+		if ( ! is_dir( $path ) ) {
 			return false;
+		}
 
 		if ( $extensions ) {
 			$extensions = (array) $extensions;
@@ -1062,18 +1114,30 @@ final class WP_Theme implements ArrayAccess {
 		}
 
 		$relative_path = trailingslashit( $relative_path );
-		if ( '/' == $relative_path )
+		if ( '/' == $relative_path ) {
 			$relative_path = '';
+		}
 
 		$results = scandir( $path );
 		$files = array();
 
+		/**
+		 * Filters the array of excluded directories and files while scanning theme folder.
+		 *
+		 * @since 4.7.4
+		 *
+		 * @param array $exclusions Array of excluded directories and files.
+		 */
+		$exclusions = (array) apply_filters( 'theme_scandir_exclusions', array( 'CVS', 'node_modules', 'vendor', 'bower_components' ) );
+
 		foreach ( $results as $result ) {
-			if ( '.' == $result[0] )
+			if ( '.' == $result[0] || in_array( $result, $exclusions, true ) ) {
 				continue;
+			}
 			if ( is_dir( $path . '/' . $result ) ) {
-				if ( ! $depth || 'CVS' == $result )
+				if ( ! $depth ) {
 					continue;
+				}
 				$found = self::scandir( $path . '/' . $result, $extensions, $depth - 1 , $relative_path . $result );
 				$files = array_merge_recursive( $files, $found );
 			} elseif ( ! $extensions || preg_match( '~\.(' . $_extensions . ')$~', $result ) ) {
@@ -1091,7 +1155,6 @@ final class WP_Theme implements ArrayAccess {
 	 * child theme, it should probably try to load the parent theme's translations.
 	 *
 	 * @since 3.4.0
-	 * @access public
 	 *
 	 * @return bool True if the textdomain was successfully loaded or has already been loaded.
 	 * 	False if no textdomain was specified in the file headers, or if the domain could not be loaded.
@@ -1125,11 +1188,10 @@ final class WP_Theme implements ArrayAccess {
 	 * Whether the theme is allowed (multisite only).
 	 *
 	 * @since 3.4.0
-	 * @access public
 	 *
 	 * @param string $check Optional. Whether to check only the 'network'-wide settings, the 'site'
 	 * 	settings, or 'both'. Defaults to 'both'.
-	 * @param int $blog_id Optional. Ignored if only network-wide settings are checked. Defaults to current blog.
+	 * @param int $blog_id Optional. Ignored if only network-wide settings are checked. Defaults to current site.
 	 * @return bool Whether the theme is allowed for the network. Returns true in single-site.
 	 */
 	public function is_allowed( $check = 'both', $blog_id = null ) {
@@ -1156,6 +1218,8 @@ final class WP_Theme implements ArrayAccess {
 	 *
 	 * This hits the filesystem.
 	 *
+	 * @since  4.4.0
+	 *
 	 * @return WP_Theme|false Object, or false if no theme is installed, which would be bad.
 	 */
 	public static function get_core_default_theme() {
@@ -1174,20 +1238,23 @@ final class WP_Theme implements ArrayAccess {
 	 * @since 3.4.0
 	 *
 	 * @static
-	 * @access public
 	 *
-	 * @param int $blog_id Optional. Defaults to current blog.
+	 * @param int $blog_id Optional. ID of the site. Defaults to the current site.
 	 * @return array Array of stylesheet names.
 	 */
 	public static function get_allowed( $blog_id = null ) {
 		/**
-		 * Filter the array of themes allowed on the site or network.
+		 * Filters the array of themes allowed on the network.
 		 *
-		 * @since MU
+		 * Site is provided as context so that a list of network allowed themes can
+		 * be filtered further.
+		 *
+		 * @since 4.5.0
 		 *
 		 * @param array $allowed_themes An array of theme stylesheet names.
+		 * @param int   $blog_id        ID of the site.
 		 */
-		$network = (array) apply_filters( 'allowed_themes', self::get_allowed_on_network() );
+		$network = (array) apply_filters( 'network_allowed_themes', self::get_allowed_on_network(), $blog_id );
 		return $network + self::get_allowed_on_site( $blog_id );
 	}
 
@@ -1197,7 +1264,6 @@ final class WP_Theme implements ArrayAccess {
 	 * @since 3.4.0
 	 *
 	 * @static
-	 * @access public
 	 *
 	 * @staticvar array $allowed_themes
 	 *
@@ -1205,8 +1271,19 @@ final class WP_Theme implements ArrayAccess {
 	 */
 	public static function get_allowed_on_network() {
 		static $allowed_themes;
-		if ( ! isset( $allowed_themes ) )
+		if ( ! isset( $allowed_themes ) ) {
 			$allowed_themes = (array) get_site_option( 'allowedthemes' );
+		}
+
+		/**
+		 * Filters the array of themes allowed on the network.
+		 *
+		 * @since MU (3.0.0)
+		 *
+		 * @param array $allowed_themes An array of theme stylesheet names.
+		 */
+		$allowed_themes = apply_filters( 'allowed_themes', $allowed_themes );
+
 		return $allowed_themes;
 	}
 
@@ -1216,11 +1293,10 @@ final class WP_Theme implements ArrayAccess {
 	 * @since 3.4.0
 	 *
 	 * @static
-	 * @access public
 	 *
 	 * @staticvar array $allowed_themes
 	 *
-	 * @param int $blog_id Optional. Defaults to current blog.
+	 * @param int $blog_id Optional. ID of the site. Defaults to the current site.
 	 * @return array Array of stylesheet names.
 	 */
 	public static function get_allowed_on_site( $blog_id = null ) {
@@ -1229,8 +1305,17 @@ final class WP_Theme implements ArrayAccess {
 		if ( ! $blog_id || ! is_multisite() )
 			$blog_id = get_current_blog_id();
 
-		if ( isset( $allowed_themes[ $blog_id ] ) )
-			return $allowed_themes[ $blog_id ];
+		if ( isset( $allowed_themes[ $blog_id ] ) ) {
+			/**
+			 * Filters the array of themes allowed on the site.
+			 *
+			 * @since 4.5.0
+			 *
+			 * @param array $allowed_themes An array of theme stylesheet names.
+			 * @param int   $blog_id        ID of the site. Defaults to current site.
+			 */
+			return (array) apply_filters( 'site_allowed_themes', $allowed_themes[ $blog_id ], $blog_id );
+		}
 
 		$current = $blog_id == get_current_blog_id();
 
@@ -1278,19 +1363,73 @@ final class WP_Theme implements ArrayAccess {
 			}
 		}
 
-		return (array) $allowed_themes[ $blog_id ];
+		/** This filter is documented in wp-includes/class-wp-theme.php */
+		return (array) apply_filters( 'site_allowed_themes', $allowed_themes[ $blog_id ], $blog_id );
 	}
 
 	/**
-	 * Sort themes by name.
+	 * Enables a theme for all sites on the current network.
+	 *
+	 * @since 4.6.0
+	 * @static
+	 *
+	 * @param string|array $stylesheets Stylesheet name or array of stylesheet names.
+	 */
+	public static function network_enable_theme( $stylesheets ) {
+		if ( ! is_multisite() ) {
+			return;
+		}
+
+		if ( ! is_array( $stylesheets ) ) {
+			$stylesheets = array( $stylesheets );
+		}
+
+		$allowed_themes = get_site_option( 'allowedthemes' );
+		foreach ( $stylesheets as $stylesheet ) {
+			$allowed_themes[ $stylesheet ] = true;
+		}
+
+		update_site_option( 'allowedthemes', $allowed_themes );
+	}
+
+	/**
+	 * Disables a theme for all sites on the current network.
+	 *
+	 * @since 4.6.0
+	 * @static
+	 *
+	 * @param string|array $stylesheets Stylesheet name or array of stylesheet names.
+	 */
+	public static function network_disable_theme( $stylesheets ) {
+		if ( ! is_multisite() ) {
+			return;
+		}
+
+		if ( ! is_array( $stylesheets ) ) {
+			$stylesheets = array( $stylesheets );
+		}
+
+		$allowed_themes = get_site_option( 'allowedthemes' );
+		foreach ( $stylesheets as $stylesheet ) {
+			if ( isset( $allowed_themes[ $stylesheet ] ) ) {
+				unset( $allowed_themes[ $stylesheet ] );
+			}
+		}
+
+		update_site_option( 'allowedthemes', $allowed_themes );
+	}
+
+	/**
+	 * Sorts themes by name.
 	 *
 	 * @since 3.4.0
 	 *
 	 * @static
-	 * @access public
+	 *
+	 * @param array $themes Array of themes to sort (passed by reference).
 	 */
 	public static function sort_by_name( &$themes ) {
-		if ( 0 === strpos( get_locale(), 'en_' ) ) {
+		if ( 0 === strpos( get_user_locale(), 'en_' ) ) {
 			uasort( $themes, array( 'WP_Theme', '_name_sort' ) );
 		} else {
 			uasort( $themes, array( 'WP_Theme', '_name_sort_i18n' ) );
@@ -1306,9 +1445,11 @@ final class WP_Theme implements ArrayAccess {
 	 * @since 3.4.0
 	 *
 	 * @static
-	 * @access private
 	 *
-	 * @return int
+	 * @param string $a First name.
+	 * @param string $b Second name.
+	 * @return int Negative if `$a` falls lower in the natural order than `$b`. Zero if they fall equally.
+	 *             Greater than 0 if `$a` falls higher in the natural order than `$b`. Used with usort().
 	 */
 	private static function _name_sort( $a, $b ) {
 		return strnatcasecmp( $a->headers['Name'], $b->headers['Name'] );
@@ -1320,9 +1461,11 @@ final class WP_Theme implements ArrayAccess {
 	 * @since 3.4.0
 	 *
 	 * @static
-	 * @access private
 	 *
-	 * @return int
+	 * @param string $a First name.
+	 * @param string $b Second name.
+	 * @return int Negative if `$a` falls lower in the natural order than `$b`. Zero if they fall equally.
+	 *             Greater than 0 if `$a` falls higher in the natural order than `$b`. Used with usort().
 	 */
 	private static function _name_sort_i18n( $a, $b ) {
 		// Don't mark up; Do translate.
