@@ -27,9 +27,13 @@
  * @property string $user_registered
  * @property string $user_activation_key
  * @property string $user_status
+ * @property int    $user_level
  * @property string $display_name
  * @property string $spam
  * @property string $deleted
+ * @property string $locale
+ * @property string $rich_editing
+ * @property string $syntax_highlighting
  */
 class WP_User {
 	/**
@@ -44,7 +48,6 @@ class WP_User {
 	 * The user's ID.
 	 *
 	 * @since 2.1.0
-	 * @access public
 	 * @var int
 	 */
 	public $ID = 0;
@@ -53,7 +56,6 @@ class WP_User {
 	 * The individual capabilities the user has been given.
 	 *
 	 * @since 2.0.0
-	 * @access public
 	 * @var array
 	 */
 	public $caps = array();
@@ -62,7 +64,6 @@ class WP_User {
 	 * User metadata option name.
 	 *
 	 * @since 2.0.0
-	 * @access public
 	 * @var string
 	 */
 	public $cap_key;
@@ -71,7 +72,6 @@ class WP_User {
 	 * The roles the user is part of.
 	 *
 	 * @since 2.0.0
-	 * @access public
 	 * @var array
 	 */
 	public $roles = array();
@@ -80,7 +80,6 @@ class WP_User {
 	 * All capabilities the user has, including individual and role based.
 	 *
 	 * @since 2.0.0
-	 * @access public
 	 * @var array
 	 */
 	public $allcaps = array();
@@ -89,14 +88,21 @@ class WP_User {
 	 * The filter context applied to user data fields.
 	 *
 	 * @since 2.9.0
-	 * @access private
 	 * @var string
 	 */
-	var $filter = null;
+	public $filter = null;
+
+	/**
+	 * The site ID the capabilities of this user are initialized for.
+	 *
+	 * @since 4.9.0
+	 * @var int
+	 */
+	private $site_id = 0;
 
 	/**
 	 * @static
-	 * @access private
+	 * @since 3.3.0
 	 * @var array
 	 */
 	private static $back_compat_keys;
@@ -107,15 +113,14 @@ class WP_User {
 	 * Retrieves the userdata and passes it to WP_User::init().
 	 *
 	 * @since 2.0.0
-	 * @access public
 	 *
 	 * @global wpdb $wpdb WordPress database abstraction object.
 	 *
 	 * @param int|string|stdClass|WP_User $id User's ID, a WP_User object, or a user object from the DB.
 	 * @param string $name Optional. User's username
-	 * @param int $blog_id Optional Blog ID, defaults to current blog.
+	 * @param int $site_id Optional Site ID, defaults to current site.
 	 */
-	public function __construct( $id = 0, $name = '', $blog_id = '' ) {
+	public function __construct( $id = 0, $name = '', $site_id = '' ) {
 		if ( ! isset( self::$back_compat_keys ) ) {
 			$prefix = $GLOBALS['wpdb']->prefix;
 			self::$back_compat_keys = array(
@@ -129,10 +134,10 @@ class WP_User {
 		}
 
 		if ( $id instanceof WP_User ) {
-			$this->init( $id->data, $blog_id );
+			$this->init( $id->data, $site_id );
 			return;
 		} elseif ( is_object( $id ) ) {
-			$this->init( $id, $blog_id );
+			$this->init( $id, $site_id );
 			return;
 		}
 
@@ -148,7 +153,7 @@ class WP_User {
 		}
 
 		if ( $data ) {
-			$this->init( $data, $blog_id );
+			$this->init( $data, $site_id );
 		} else {
 			$this->data = new stdClass;
 		}
@@ -157,14 +162,16 @@ class WP_User {
 	/**
 	 * Sets up object properties, including capabilities.
 	 *
-	 * @param object $data User DB row object
-	 * @param int $blog_id Optional. The blog id to initialize for
+	 * @since  3.3.0
+	 *
+	 * @param object $data    User DB row object.
+	 * @param int    $site_id Optional. The site ID to initialize for.
 	 */
-	public function init( $data, $blog_id = '' ) {
+	public function init( $data, $site_id = '' ) {
 		$this->data = $data;
 		$this->ID = (int) $data->ID;
 
-		$this->for_blog( $blog_id );
+		$this->for_site( $site_id );
 	}
 
 	/**
@@ -241,35 +248,17 @@ class WP_User {
 		return $user;
 	}
 
-	/**
-	 * Makes private/protected methods readable for backwards compatibility.
-	 *
-	 * @since 4.3.0
-	 * @access public
-	 *
-	 * @param callable $name      Method to call.
-	 * @param array    $arguments Arguments to pass when calling.
-	 * @return mixed|false Return value of the callback, false otherwise.
-	 */
-	public function __call( $name, $arguments ) {
-		if ( '_init_caps' === $name ) {
-			return call_user_func_array( array( $this, $name ), $arguments );
-		}
-		return false;
-	}
-
 	/**
 	 * Magic method for checking the existence of a certain custom field.
 	 *
 	 * @since 3.3.0
-	 * @access public
 	 *
 	 * @param string $key User meta key to check if set.
 	 * @return bool Whether the given user meta key is set.
 	 */
 	public function __isset( $key ) {
 		if ( 'id' == $key ) {
-			_deprecated_argument( 'WP_User->id', '2.1',
+			_deprecated_argument( 'WP_User->id', '2.1.0',
 				sprintf(
 					/* translators: %s: WP_User->ID */
 					__( 'Use %s instead.' ),
@@ -292,14 +281,13 @@ class WP_User {
 	 * Magic method for accessing custom fields.
 	 *
 	 * @since 3.3.0
-	 * @access public
 	 *
 	 * @param string $key User meta key to retrieve.
 	 * @return mixed Value of the given user meta key (if set). If `$key` is 'id', the user ID.
 	 */
 	public function __get( $key ) {
 		if ( 'id' == $key ) {
-			_deprecated_argument( 'WP_User->id', '2.1',
+			_deprecated_argument( 'WP_User->id', '2.1.0',
 				sprintf(
 					/* translators: %s: WP_User->ID */
 					__( 'Use %s instead.' ),
@@ -331,14 +319,13 @@ class WP_User {
 	 * the value on the WP_User instance.
 	 *
 	 * @since 3.3.0
-	 * @access public
 	 *
 	 * @param string $key   User meta key.
 	 * @param mixed  $value User meta value.
 	 */
 	public function __set( $key, $value ) {
 		if ( 'id' == $key ) {
-			_deprecated_argument( 'WP_User->id', '2.1',
+			_deprecated_argument( 'WP_User->id', '2.1.0',
 				sprintf(
 					/* translators: %s: WP_User->ID */
 					__( 'Use %s instead.' ),
@@ -356,13 +343,12 @@ class WP_User {
 	 * Magic method for unsetting a certain custom field.
 	 *
 	 * @since 4.4.0
-	 * @access public
 	 *
 	 * @param string $key User meta key to unset.
 	 */
 	public function __unset( $key ) {
 		if ( 'id' == $key ) {
-			_deprecated_argument( 'WP_User->id', '2.1',
+			_deprecated_argument( 'WP_User->id', '2.1.0',
 				sprintf(
 					/* translators: %s: WP_User->ID */
 					__( 'Use %s instead.' ),
@@ -384,7 +370,6 @@ class WP_User {
 	 * Determine whether the user exists in the database.
 	 *
 	 * @since 3.4.0
-	 * @access public
 	 *
 	 * @return bool True if user exists in the database, false if not.
 	 */
@@ -431,6 +416,22 @@ class WP_User {
 		return get_object_vars( $this->data );
 	}
 
+	/**
+	 * Makes private/protected methods readable for backward compatibility.
+	 *
+	 * @since 4.3.0
+	 *
+	 * @param callable $name      Method to call.
+	 * @param array    $arguments Arguments to pass when calling.
+	 * @return mixed|false Return value of the callback, false otherwise.
+	 */
+	public function __call( $name, $arguments ) {
+		if ( '_init_caps' === $name ) {
+			return call_user_func_array( array( $this, $name ), $arguments );
+		}
+		return false;
+	}
+
 	/**
 	 * Set up capability object properties.
 	 *
@@ -439,8 +440,8 @@ class WP_User {
 	 * property matching the 'cap_key' exists and is an array. If so, it will be
 	 * used.
 	 *
-	 * @access protected
 	 * @since 2.1.0
+	 * @deprecated 4.9.0 Use WP_User::for_site()
 	 *
 	 * @global wpdb $wpdb WordPress database abstraction object.
 	 *
@@ -449,15 +450,15 @@ class WP_User {
 	protected function _init_caps( $cap_key = '' ) {
 		global $wpdb;
 
-		if ( empty($cap_key) )
-			$this->cap_key = $wpdb->get_blog_prefix() . 'capabilities';
-		else
+		_deprecated_function( __METHOD__, '4.9.0', 'WP_User::for_site()' );
+
+		if ( empty( $cap_key ) ) {
+			$this->cap_key = $wpdb->get_blog_prefix( $this->site_id ) . 'capabilities';
+		} else {
 			$this->cap_key = $cap_key;
+		}
 
-		$this->caps = get_user_meta( $this->ID, $this->cap_key, true );
-
-		if ( ! is_array( $this->caps ) )
-			$this->caps = array();
+		$this->caps = $this->get_caps_data();
 
 		$this->get_role_caps();
 	}
@@ -471,11 +472,17 @@ class WP_User {
 	 * granted permission to.
 	 *
 	 * @since 2.0.0
-	 * @access public
 	 *
 	 * @return array List of all capabilities for the user.
 	 */
 	public function get_role_caps() {
+		$switch_site = false;
+		if ( is_multisite() && $this->site_id != get_current_blog_id() ) {
+			$switch_site = true;
+
+			switch_to_blog( $this->site_id );
+		}
+
 		$wp_roles = wp_roles();
 
 		//Filter out caps that are not role names and assign to $this->roles
@@ -490,6 +497,10 @@ class WP_User {
 		}
 		$this->allcaps = array_merge( (array) $this->allcaps, (array) $this->caps );
 
+		if ( $switch_site ) {
+			restore_current_blog();
+		}
+
 		return $this->allcaps;
 	}
 
@@ -499,7 +510,6 @@ class WP_User {
 	 * Updates the user's meta data option with capabilities and roles.
 	 *
 	 * @since 2.0.0
-	 * @access public
 	 *
 	 * @param string $role Role name.
 	 */
@@ -528,7 +538,6 @@ class WP_User {
 	 * Remove role from user.
 	 *
 	 * @since 2.0.0
-	 * @access public
 	 *
 	 * @param string $role Role name.
 	 */
@@ -559,7 +568,6 @@ class WP_User {
 	 * of the roles from the user.
 	 *
 	 * @since 2.0.0
-	 * @access public
 	 *
 	 * @param string $role Role name.
 	 */
@@ -607,7 +615,6 @@ class WP_User {
 	 * value.
 	 *
 	 * @since 2.0.0
-	 * @access public
 	 *
 	 * @param int $max Max level of user.
 	 * @param string $item Level capability name.
@@ -630,7 +637,6 @@ class WP_User {
 	 * the all of the capabilities that the user has.
 	 *
 	 * @since 2.0.0
-	 * @access public
 	 *
 	 * @global wpdb $wpdb WordPress database abstraction object.
 	 */
@@ -644,7 +650,6 @@ class WP_User {
 	 * Add capability and grant or deny access to capability.
 	 *
 	 * @since 2.0.0
-	 * @access public
 	 *
 	 * @param string $cap Capability name.
 	 * @param bool $grant Whether to grant capability to user.
@@ -660,7 +665,6 @@ class WP_User {
 	 * Remove capability from user.
 	 *
 	 * @since 2.0.0
-	 * @access public
 	 *
 	 * @param string $cap Capability name.
 	 */
@@ -678,7 +682,6 @@ class WP_User {
 	 * Remove all of the capabilities of the user.
 	 *
 	 * @since 2.1.0
-	 * @access public
 	 *
 	 * @global wpdb $wpdb WordPress database abstraction object.
 	 */
@@ -691,28 +694,26 @@ class WP_User {
 	}
 
 	/**
-	 * Whether user has capability or role name.
+	 * Whether the user has a specific capability.
 	 *
-	 * While checking against particular roles in place of a capability is supported
-	 * in part, this practice is discouraged as it may produce unreliable results.
+	 * While checking against a role in place of a capability is supported in part, this practice is discouraged as it
+	 * may produce unreliable results.
 	 *
 	 * @since 2.0.0
-	 * @access public
 	 *
 	 * @see map_meta_cap()
 	 *
-	 * @param string $cap       Capability name.
-	 * @param int    $object_id Optional. ID of the specific object to check against if `$cap` is a "meta" cap.
-	 *                          "Meta" capabilities, e.g. 'edit_post', 'edit_user', etc., are capabilities used
-	 *                          by map_meta_cap() to map to other "primitive" capabilities, e.g. 'edit_posts',
-	 *                          'edit_others_posts', etc. The parameter is accessed via func_get_args() and passed
-	 *                          to map_meta_cap().
-	 * @return bool Whether the current user has the given capability. If `$cap` is a meta cap and `$object_id` is
-	 *              passed, whether the current user has the given meta capability for the given object.
+	 * @param string $cap           Capability name.
+	 * @param int    $object_id,... Optional. ID of a specific object to check against if `$cap` is a "meta" capability.
+	 *                              Meta capabilities such as `edit_post` and `edit_user` are capabilities used by
+	 *                              by the `map_meta_cap()` function to map to primitive capabilities that a user or
+	 *                              role has, such as `edit_posts` and `edit_others_posts`.
+	 * @return bool Whether the user has the given capability, or, if `$object_id` is passed, whether the user has
+	 *              the given capability for that object.
 	 */
 	public function has_cap( $cap ) {
 		if ( is_numeric( $cap ) ) {
-			_deprecated_argument( __FUNCTION__, '2.0', __('Usage of user levels by plugins and themes is deprecated. Use roles and capabilities instead.') );
+			_deprecated_argument( __FUNCTION__, '2.0.0', __( 'Usage of user levels is deprecated. Use capabilities instead.' ) );
 			$cap = $this->translate_level_to_cap( $cap );
 		}
 
@@ -743,6 +744,9 @@ class WP_User {
 		// Everyone is allowed to exist.
 		$capabilities['exist'] = true;
 
+		// Nobody is allowed to do things they are not allowed to do.
+		unset( $capabilities['do_not_allow'] );
+
 		// Must have ALL requested caps.
 		foreach ( (array) $caps as $cap ) {
 			if ( empty( $capabilities[ $cap ] ) )
@@ -758,7 +762,6 @@ class WP_User {
 	 * Prepends 'level_' to level number.
 	 *
 	 * @since 2.0.0
-	 * @access public
 	 *
 	 * @param int $level Level number, 1 to 10.
 	 * @return string
@@ -768,20 +771,71 @@ class WP_User {
 	}
 
 	/**
-	 * Set the blog to operate on. Defaults to the current blog.
+	 * Set the site to operate on. Defaults to the current site.
 	 *
 	 * @since 3.0.0
+	 * @deprecated 4.9.0 Use WP_User::for_site()
 	 *
 	 * @global wpdb $wpdb WordPress database abstraction object.
 	 *
-	 * @param int $blog_id Optional Blog ID, defaults to current blog.
+	 * @param int $blog_id Optional. Site ID, defaults to current site.
 	 */
 	public function for_blog( $blog_id = '' ) {
+		_deprecated_function( __METHOD__, '4.9.0', 'WP_User::for_site()' );
+
+		$this->for_site( $blog_id );
+	}
+
+	/**
+	 * Sets the site to operate on. Defaults to the current site.
+	 *
+	 * @since 4.9.0
+	 *
+	 * @global wpdb $wpdb WordPress database abstraction object.
+	 *
+	 * @param int $site_id Site ID to initialize user capabilities for. Default is the current site.
+	 */
+	public function for_site( $site_id = '' ) {
 		global $wpdb;
-		if ( ! empty( $blog_id ) )
-			$cap_key = $wpdb->get_blog_prefix( $blog_id ) . 'capabilities';
-		else
-			$cap_key = '';
-		$this->_init_caps( $cap_key );
+
+		if ( ! empty( $site_id ) ) {
+			$this->site_id = absint( $site_id );
+		} else {
+			$this->site_id = get_current_blog_id();
+		}
+
+		$this->cap_key = $wpdb->get_blog_prefix( $this->site_id ) . 'capabilities';
+
+		$this->caps = $this->get_caps_data();
+
+		$this->get_role_caps();
+	}
+
+	/**
+	 * Gets the ID of the site for which the user's capabilities are currently initialized.
+	 *
+	 * @since 4.9.0
+	 *
+	 * @return int Site ID.
+	 */
+	public function get_site_id() {
+		return $this->site_id;
+	}
+
+	/**
+	 * Gets the available user capabilities data.
+	 *
+	 * @since 4.9.0
+	 *
+	 * @return array User capabilities array.
+	 */
+	private function get_caps_data() {
+		$caps = get_user_meta( $this->ID, $this->cap_key, true );
+
+		if ( ! is_array( $caps ) ) {
+			return array();
+		}
+
+		return $caps;
 	}
 }
